// logandc2().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_integer.h"


// Implementation.

#include "cl_I.h"
#include "cl_DS.h"
#include "cl_I_log.h"

// Logische Operationen auf Integers:
// Methode: aus den Lngen der beiden Argumente eine obere Schranke fr
// die Lnge des Ergebnisses berechnen (das Maximum der beiden Lngen und
// FN_maxlength), so da das MSD fr unendlich viele Bits steht.
// Dann beide Argumente in gleichgroe Digit sequences umwandeln, Operation
// mit einer einfachen Schleife durchfhren.

const cl_I logandc2 (const cl_I& x, const cl_I& y)
    { if (fixnump(x) && fixnump(y)) // Beides Fixnums -> ganz einfach:
        { // bitweise als Fixnum zurck
          return cl_I_from_word((x.word & ~ y.word) | cl_combine(cl_FN_tag,0));
        }
      if (fixnump(x))
        { DeclareType(cl_FN,x);
          if (!minusp(x))
            // PosFixnum AND Bignum -> PosFixnum
            { return cl_I_from_word(x.word & ~ cl_combine(0,pFN_maxlength_digits_at(BN_LSDptr(y)))); }
        }
      { CL_ALLOCA_STACK;
        var uintC n; // Anzahl der Digits
       {var uintC nx = I_to_DS_need(x);
        var uintC ny = I_to_DS_need(y);
        n = (nx>=ny ? nx : ny);
       }
       {var uintD* xptr; I_to_DS_n(x,n,xptr=); // Pointer in DS zu x
        var uintD* yptr; I_to_DS_n(y,n,yptr=); // Pointer in DS zu y
        var uintD* zptr = xptr; // Pointer aufs Ergebnis
        andc2_loop_msp(xptr,yptr,n); // mit AND NOT verknpfen
        return DS_to_I(zptr,n); // Ergebnis als Integer
    } }}
